package ads1ws08.pa1;

import java.util.ArrayList;
import java.util.Iterator;

/**
 * Eine Klasse, die die grundlegenden Operationen der Datenstruktur
 * AVL-Baum implementiert.
 *
 * <p>
 * Von den drei wichtigsten Operationen Einf&uuml;gen, Suchen und
 * Entfernen, sind {@link #insert Einf&uuml;gen} und {@link #find Suchen}
 * bereits implementiert. Ihre Aufgabe ist es das {@link #remove Entfernen}
 * eines Elements zu implementieren (in der Klasse {@link AvlTreeImpl}),
 * wof&uuml;r Sie auch die Methoden {@link #rotateLeft rotateLeft()} und
 * {@link #rotateRight rotateRight()} implementieren m&uuml;ssen.
 * </p>
 *
 * <p>
 * Die restlichen Methoden dieser Klasse vervollst&auml;ndigen nur das
 * Interface eines AVL-Baums, oder sind f&uuml;r Debugging-Informationen
 * zust&auml;ndig.
 * </p>
 *
 * <p>
 * <b>WICHTIG:</b>Nehmen Sie keine &Auml;nderungen in dieser Klasse vor.
 * Bei der Abgabe werden diese &Auml;nderungen verworfen und es k&ouml;nnte
 * dadurch passieren, dass Ihr Programm somit nicht mehr korrekt
 * funktioniert.
 * </p>
 */
public abstract class AbstractAvlTree
{

/**
 * Der Wurzel-Knoten des AVL-Baums.
 *
 * Bei einem leeren AVL-Baum gilt <code>root.isNull == true</code>.
 */
protected AvlNode root;

/** Die Anzahl an gespeicherten Elementen im AVL-Baum. */
protected int size;

private boolean firstDebugMsg;

private final String NOERRORS = "";
private final String HEIGHT = "height is invalid";
private final String BALANCE = "balance is invalid";
private final String SEARCHTREE = "no searchtree";
//private final static String NULLPOINTER ="nullpointer exception";

private String errorCode = NOERRORS;

/**
 * Erzeugt einen neuen leeren AVL-Baum.
 */


public
AbstractAvlTree()
{
	root = AvlNode.createRootNode();
	size = 0;
	firstDebugMsg = true;
}

/**
 * F&uuml;hrt eine Linksrotation beim Knoten <code>n</code> durch.
 *
 * Diese Methode muss in {@link AvlTreeImpl#rotateLeft AvlTreeImpl.roateLeft()}
 * implementiert werden.
 *
 * @param n Der Knoten der nach links rotiert werden soll.
 *
 * @return Der neue Wurzelknoten des rotierten Teilbaums.
 */
public abstract AvlNode rotateLeft( AvlNode n );

/**
 * F&uuml;hrt eine Rechtsrotation beim Knoten <code>n</code> durch.
 *
 * Diese Methode muss in
 * {@link AvlTreeImpl#rotateRight AvlTreeImpl.rotateRight()} implementiert
 * werden.
 *
 * @param n Der Knoten der nach rechts rotiert werden soll.
 *
 * @return Der neue Wurzelknoten des rotierten Teilbaums.
 */
public abstract AvlNode rotateRight( AvlNode n );


/**
 * Entfernt den Knoten mit Schl&uuml;ssel <code>k</code> aus dem
 * AVL-Baum.
 *
 * @param k Der Schl&uuml;ssel des Knotens der entfernt werden soll.
 */
public abstract void remove( int k ) throws Exception;

/**
 * Liefert die Anzahl an gespeichterten Elementen im AVL-Baum.
 *
 * @return Die Anzahl an gespeicherten Elements im AVL-Baum.
 *
 * @see #size
 */
public int
getSize()
{
	return size;
}

/**
 * Liefert das kleinste Element im AVL-Baum, oder {@link Integer#MIN_VALUE},
 * falls der Baum leer ist.
 *
 * @return Den kleinsten gespeicherten Schl&uuml;ssel falls der Baum
 * 	nicht leer ist, sonst <code>Integer.MIN_VALUE</code>.
 */
public int
getMin() throws Exception
{
	if (root.isNull()) return Integer.MIN_VALUE;

	AvlNode n = root;

	while (!root.left.isNull()) n = n.left;

	return n.getKey();
}

/**
 * Liefert das gr&ouml;&szlig;te Element im AVL-Baum, oder
 * {@link Integer#MAX_VALUE}, falls der Baum leer ist.
 *
 * @return Den gr&ouml;&szlig;ten gespeicherten Schl&uuml;ssel falls der Baum
 * 	nicht leer ist, sonst <code>Integer.MAX_VALUE</code>.
 */
public int
getMax() throws Exception
{
	if (root.isNull()) return Integer.MAX_VALUE;

	AvlNode n = root;

	while (!root.right.isNull()) n = n.right;

	return n.getKey();
}

/**
 * Sucht den Knoten mit dem Schl&uuml;ssel <code>k</code>.
 *
 * @param k Der Schl&uuml;ssel der im Baum gesucht werden soll.
 *
 * @return Den {@link AvlNode Knoten} mit dem Schl&uuml;ssel
 * <code>k</code>, falls er im AVL-Baum enthalten ist, sonst einen
 * Knoten <code>n</code> mit <code>n.isNull == true</code>.
 */
public AvlNode
find( int k ) throws Exception
{
	AvlNode n = root;

	while (!n.isNull()) {
		if (k < n.getKey()) n = n.left;
		else if (k > n.getKey()) n = n.right;
		else break;
	}

	return n;
}

/**
 * F&uuml;gt ein Element mit dem Schl&uuml;ssel <code>k</code> ein.
 *
 * <p>
 * <b>HINWEIS:</b> Die Implementierung dieser Methode verwendet
 * {@link AvlTreeImpl#rotateLeft} und {@link AvlTreeImpl#rotateRight},
 * die von Ihnen implementiert werden m&uuml;ssen. Eine fehlerhafte
 * implementierung von <code>rotateLeft</code> oder <code>rotateRight</code>
 * f&uuml;hrt also mit ziemlicher Sicherheit zu einem fehlerhaften
 * Einf&uuml;gen.
 * </p>
 *
 * @param k Der Schl&uuml;ssel der eingef&uuml;gt werden soll.
 * 	Ist bereits ein Element mit dem Schl&uuml;ssel <code>k</code> im
 * 	Baum enthalten, macht diese Methode nichts.
 */
public void
insert( int k ) throws Exception
{
	AvlNode oldPar = root.parent;

	root = insert(root, k);
	root.parent = oldPar;
}

private AvlNode
insert( AvlNode n, int k ) throws Exception
{
	if (n.isNull()) {
		n.setKey(k);
		size++;

		return n;
	}

	if (n.getKey() == k) return n;

	if (k < n.getKey()) {
		n.left = insert(n.left, k);
		n.left.parent = n;
		n.height = 1 + (n.left.height > n.right.height
			? n.left.height : n.right.height);
		n.balance = n.right.height - n.left.height;
		if (n.right.height - n.left.height == -2) {
			if (n.left.left.height >= n.left.right.height) {
				n = rotateRight(n);
			} else {
				n.left = rotateLeft(n.left);
				n = rotateRight(n);
			}
		}
	} else {
		n.right = insert(n.right, k);
		n.right.parent = n;
		n.height = 1 + (n.left.height > n.right.height
			? n.left.height : n.right.height);
		n.balance = n.right.height - n.left.height;
		if (n.right.height - n.left.height == 2) {
			if (n.right.right.height >= n.right.left.height) {
				n = rotateLeft(n);
			} else {
				n.right = rotateRight(n.right);
				n = rotateLeft(n);
			}
		}
	}

	return n;
}

private ArrayList<AvlNode>
inorder()
{
	ArrayList<AvlNode> ret = new ArrayList<AvlNode>();

	inorder(root, ret);

	return ret;
}

private void
inorder( AvlNode n, ArrayList<AvlNode> io )
{
	if (n == null || n.isNull()) return;

	inorder(n.left, io);

	io.add(n);

	inorder(n.right, io);
}

private ArrayList<AvlNode>
preorder()
{
	ArrayList<AvlNode> ret = new ArrayList<AvlNode>();

	preorder(root, ret);

	return ret;
}

private void
preorder( AvlNode n, ArrayList<AvlNode> po )
{
	if (n == null || n.isNull()) return;

	po.add(n);

	preorder(n.left, po);
	preorder(n.right, po);
}

/**
 * Gibt den aktuellen Zustand des Baums auf {@link System#out} aus.
 *
 * <p>
 * Wird die Methode zum ersten Mal aufgerufen, wird auch eine Warnung
 * ausgegeben, dass alle Aufrufe von <code>printDebug()</code> vor der
 * Abgabe entfernt werden m&uuml;ssen.
 * </p>
 *
 * <p>
 * Durch das Setzen von {@link AvlNode#debugLevel} kann der Detailgrad
 * der Debugging-Ausgaben bestimmt werden.
 * </p>
 *
 * @see AvlNode#setDebugLevel
 */
public void
printDebug()
{
	if (firstDebugMsg) {
		System.out.println(
			"#warning YOU HAVE TO REMOVE ALL CALLS TO "
			+ "printDebug() BEFORE YOU SUBMIT YOUR PROGRAM!");
		firstDebugMsg = false;
	}

	System.out.println(getTree());
}

/**
 * Liefert eine f&uuml;r die Ausgabe formatierte
 * String-Repr&auml;sentation des AVL-Baums.
 *
 * Der {@link AvlNode#debugLevel Debug-Level} wird dabei ignoriert.
 */
public String
getResult()
{
	String ret;
	int debug = AvlNode.getDebugLevel();

	AvlNode.setDebugLevel(AvlNode.DEBUG_OFF);
	ret = "#result " + inorder().toString();
	AvlNode.setDebugLevel(debug);

	return ret;
}

private String
getTree()
{
	if (root.isNull()) return "#tree -1";

	ArrayList<AvlNode> po = preorder();
	ArrayList<AvlNode> io = inorder();
	StringBuffer pos = new StringBuffer();
	StringBuffer ios = new StringBuffer();

	assert po.size() == io.size();

	Iterator<AvlNode> lit = po.iterator();

	pos.append(lit.next());
	while (lit.hasNext()) pos.append("," + lit.next());

	lit = io.iterator();
	ios.append(lit.next());
	while (lit.hasNext()) ios.append("," + lit.next());

	return "#tree " + root.height + " " + pos + " " + ios;
}

/** Liefert eine String-Repr&auml;sentation des AVL-Baums. */
public String
toString()
{
	if (root.isNull()) return "<Empty tree>";

	String nl = System.getProperty("line.separator");

	return "[AvlTree]" + nl
		+ "   Preorder: " + preorder() + nl
		+ "   Inorder: " + inorder();
}

public String testHeightBallance() throws Exception
{
	this.errorCode = NOERRORS;

	if (root.isNull())
		return errorCode;
	
	testHeightBallanceRec(root);
	return errorCode;

}


private int testHeightBallanceRec(AvlNode n) throws Exception
{
	if (errorCode != NOERRORS)
		return -1;
	else
	{
		int height = 0;
		int lheight = 0;
		int rheight = 0;
		int bal = 0;
		
		if (n.left.isNull() == false)
		{
			if (n.left.getKey() > n.getKey())
			{
				errorCode = SEARCHTREE;
				return -1;
			}
			lheight = testHeightBallanceRec(n.left);
		}
		if (n.right.isNull() == false)
		{
			if (n.right.getKey() < n.getKey())
			{
				errorCode = SEARCHTREE;
				return -1;
			}
			rheight = testHeightBallanceRec(n.right);
		}
		
		if (errorCode != NOERRORS)
			return -1;
		
		height = Math.max(lheight, rheight);
		bal = rheight - lheight;
		
		if (height != n.height)
		{
			errorCode = HEIGHT;
		}
		if (bal > 1 || bal < -1)
		{
			errorCode = BALANCE;
		}
		return ++height;
	}
}

} // class AbstractAvlTree
