package ads1ws08.pa1;

/**
 * Diese Klasse kapselt alle Informationen eines Knoten des AVL-Baums.
 *
 * <p>
 * Die Variablen {@link #parent}, {@link #left}, {@link #right},
 * {@link #height} und {@link #balance} k&ouml;nnen direkt angesprochen
 * werden. Auf {@link #key} muss mittels {@link #getKey} und
 * {@link #setKey} zugegriffen werden.
 * </p>
 *
 * <p>
 * Eine Anwendung sollte nur eine Instanz von <code>AvlNode</code>
 * erzeugen und zwar mit {@link #createRootNode}.
 * </p>
 *
 * <p>
 * Ein <code>NULL</code>-Knoten wird durch ein
 * <code>AvlNode</code>-Object <code>n</code> repr&auml;sentiert f&uuml;r
 * das <code>n.isNull == true</code> gilt.
 * </p>
 *
 * <p>
 * <em>Alle</em> Knoten eines AVL-Baums haben zwei <code>AvlNode</code>
 * Objekte als Nachfolger (<code>left</code> und <code>right</code>).
 * Wenn ein Knoten im Baum keinen linken oder rechten Nachfolger hat,
 * ist f&uuml;r das entsprechende Objekt {@link #isNull} immer
 * <code>true</code>. F&uuml;r jeden richtigen Knoten des Baums ist
 * <code>isNull</code> immer <code>false</code>.
 * </p>
 *
 * <p>
 * Wenn auf einen Knoten f&uuml;r den <code>isNull == true</code> gilt
 * <code>setKey</code> aufgerufen wird, wird aus dem
 * <code>NULL</code>-Knoten automatisch ein richtiger Knoten (und
 * erh&auml;lt dadurch zwei Kinder f&uuml;r die wiederum
 * <code>isNull == true</code> gilt).
 * </p>
 *
 * <p>
 * Wenn die <code>parent</code>-Eigentschaft gesetzt ist, muss f&uuml;r
 * die Wurzel <code>r</code> des AVL-Baumes gelten
 * <code>r.parent.isNull == true</code>.
 * </p>
 *
 * <p>
 * <b>WICHTIG:</b>Nehmen Sie keine &Auml;nderungen in dieser Klasse vor.
 * Bei der Abgabe werden diese &Auml;nderungen verworfen und es k&ouml;nnte
 * dadurch passieren, dass Ihr Programm somit nicht mehr korrekt
 * funktioniert.
 * </p>
 */
public class AvlNode
{

/** Es wird nur der {@link #key Schl&uuml;ssel} des Knotens ausgegeben. */
public static final int DEBUG_OFF = 0;

/**
 * Die Schl&uuml;ssel des {@link #left linken} und
 * {@link #right rechten} Kindes werden ebenfalls ausgegeben.
 */
public static final int DEBUG_CHILDREN = 0x01;

/**
 * Die {@link #height H&ouml;he} des Knotens wird ebenfalls ausgegeben.
 */
public static final int DEBUG_HEIGHT = 0x02;

/**
 * Die {@link #balance Balance} des Knotens wird ebenfalls ausgegeben.
 */
public static final int DEBUG_BALANCE = 0x04;

/**
 * Der Schl&uuml;ssel des {@link #parent Elternknotens} wird ebenfalls
 * ausgegeben.
 */
public static final int DEBUG_PARENT = 0x08;

/** Der von der JVM berechnete Hash des Objektes wird ebenfalls ausgegeben. */
public static final int DEBUG_ID = 0xa0;

/** Alle Informationen &uuml;ber den Knoten werden ausgegeben. */
public static final int DEBUG_ALL =
	DEBUG_CHILDREN | DEBUG_HEIGHT | DEBUG_BALANCE | DEBUG_PARENT | DEBUG_ID;

/**
 * Diese Variable steuert, welche Informationen durch
 * {@link #toString} ausgegeben werden.
 *
 * <p>
 * M&ouml;gliche Werte sind:
 * <ul>
 *   <li>{@link #DEBUG_OFF}: Nur der Schl&uuml;ssel des Knotens</li>
 *   <li>{@link #DEBUG_HEIGHT}: Die H&ouml;he des Knotens</li>
 *   <li>{@link #DEBUG_BALANCE}: Die Balance des Knotens</li>
 *   <li>{@link #DEBUG_PARENT}: Der Schl&uuml;ssel des Elternknotens</li>
 *   <li>{@link #DEBUG_CHILDREN}: Die Schl&uuml;ssel der Kindknoten</li>
 *   <li>{@link #DEBUG_ALL}: Alle Informationen</li>
 * </ul>
 * Der Default-Wert ist <code>DEBUG_OFF</code>.
 * </p>
 *
 * @see #getDebugLevel
 * @see #setDebugLevel
 */
private static int debugLevel = DEBUG_OFF;

/**
 * Der Elternknoten.
 *
 * <p>
 * Handelt es sich bei dem aktuellen Knoten <code>n</code> um die Wurzel
 * des AVL-Baumes sollte <code>n.parent.isNull == true</code> gelten (auch wenn
 * <code>n.isNull == true</code> gilt).
 * </p>
 *
 * <p>
 * <b>HINWEIS:</b> Diese Variable wird vom Framework <em>nicht</em>
 * verwendet, und muss nur korrekt gesetzt sein, wenn Sie sie f&uuml;r
 * Ihre Implementierung brauchen.
 * </p>
 *
 * @see #isNull
 */
public AvlNode parent;

/**
 * Das linke Kind.
 *
 * <p>
 * Wenn f&uuml;r den aktuellen Knoten <code>n<code> gilt, dass
 * <code>n.isNull == true</code>, kann ein Zugriff auf
 * <code>n.left</code> eine {@link java.lang.NullPointerException}
 * ausl&ouml;sen.
 * </p>
 *
 * <p>
 * <b>HINWEIS:</b> Diese Variable wird vom Framework verwendet und muss
 * nach jedem Aufruf von {@link AvlTreeImpl#remove remove()},
 * {@link AvlTreeImpl#rotateLeft rotateLeft()} und
 * {@link AvlTreeImpl#rotateRight rotateRight()} einen korrekten Wert haben.
 * </p>
 *
 * @see #isNull
 */
public AvlNode left;

/**
 * Das rechte Kind.
 *
 * Wenn f&uuml;r den aktuellen Knoten <code>n<code> gilt, dass
 * <code>n.isNull == true</code>, kann ein Zugriff auf
 * <code>n.right</code> eine {@link java.lang.NullPointerException}
 * ausl&ouml;sen.
 *
 * <p>
 * <b>HINWEIS:</b> Diese Variable wird vom Framework verwendet und muss
 * nach jedem Aufruf von {@link AvlTreeImpl#remove remove()},
 * {@link AvlTreeImpl#rotateLeft rotateLeft()} und
 * {@link AvlTreeImpl#rotateRight rotateRight()} einen korrekten Wert haben.
 * </p>
 *
 * @see #isNull
 */
public AvlNode right;

/**
 * Die h&ouml;he des Knotens.
 *
 * Gilt f&uuml;r einen Knoten <code>n</code>, dass
 * <code>n.isNull == true</code>, dann ist <code>n.height == -1</code>.
 *
 * <p>
 * <b>HINWEIS:</b> Diese Variable wird vom Framework verwendet und muss
 * nach jedem Aufruf von {@link AvlTreeImpl#remove remove()},
 * {@link AvlTreeImpl#rotateLeft rotateLeft()} und
 * {@link AvlTreeImpl#rotateRight rotateRight()} einen korrekten Wert haben.
 * </p>
 *
 * @see #isNull
 */
public int height;

/**
 * Die Balance des Knotens.
 *
 * <p>
 * Gilt f&uuml;r einen Knoten <code>n</code>, dass
 * <code>n.isNull == true</code>, dann ist <code>n.balance == 0</code>.
 * </p>
 *
 * <p>
 * <b>HINWEIS:</b> Diese Variable wird vom Framework <em>nicht</em>
 * verwendet, und muss nur korrekt gesetzt sein, wenn Sie sie f&uuml;r
 * Ihre Implementierung brauchen.
 * </p>
 *
 * @see #isNull
 */
public int balance;

/**
 * Der Schl&uuml;ssel des Knotens.
 * @see #setKey
 * @see #getKey
 */
private int key;

/**
 * Gibt an, ob dieser Knoten einen <code>NULL</code>-Knoten repr&auml;sentiert.
 *
 * <p>
 * F&uuml;r einen <code>NULL</code>-Knoten <code>n</code> muss gelten:
 * <ul>
 *   <li><code>n.height == -1</code></li>
 *   <li><code>n.balance == 0</code></li>
 *   <li><code>n.left == null</code></li>
 *   <li><code>n.right == null</code></li>
 * </ul>
 * <p>
 *
 * <p>
 * Da ein <code>NULL</code>-Knoten keinen Wert im AVL-Baum
 * repr&auml;sentiert, darf {@link #getKey} nicht f&uuml;r einen
 * <code>NULL</code>-Knoten aufgerufen werden.
 * </p>
 *
 * <p>
 * Wird (zum Beispiel im Zuge einer Einf&uuml;ge-Operation) ein
 * <code>NULL</code>-Knoten <code>n</code> durch den Aufruf von
 * <code>n.setKey()</code> zu einem regul&auml;ren Knoten, dann werden
 * {@link #height} und {@link #balance} automatisch auf 0 gesetzt und an
 * {@link #left} und {@link #right} werden zwei neue
 * <code>NULL</code>-Knoten angeh&auml;ngt.
 * </p>
 *
 * <p>
 * Die Eigenschaft <code>isNull</code> ist read-only und kann &uuml;ber
 * {@link #isNull()} abgefragt werden.
 * </p>
 *
 * @see #height
 * @see #balance
 * @see #isNull()
 * @see #setKey
 * @see #getKey
 */
private boolean isNull;

/** Der Default-Konstruktor.
 *
 * <p>
 * Erzeugt einen Wurzel-<code>NULL</code>-Knoten.
 * </p>
 *
 * <p>
 * <b>HINWEIS:</b> Alle Konstruktoren von <code>AvlNode</code> sind
 * <code>private</code>. Neue Instanzen k&ouml;nnen von einer Anwendung
 * nur &uuml;ber {@link #createRootNode} erzeugt werden, bzw. werden bei
 * einem Aufruf von {@link #setKey} automatisch erzeugt.
 * </p>
 *
 * @see #isNull
 */
private
AvlNode()
{
	this(null);
}

/**
 * Erzeugt einen <code>NULL</code>-Knoten als Kind von <code>p</code>.
 *
 * <p>
 * <b>HINWEIS:</b> Alle Konstruktoren von <code>AvlNode</code> sind
 * <code>private</code>. Neue Instanzen k&ouml;nnen von einer Anwendung
 * nur &uuml;ber {@link #createRootNode} erzeugt werden, bzw. werden bei
 * einem Aufruf von {@link #setKey} automatisch erzeugt.
 * </p>
 *
 * @param p Der Elternknoten des neuen Knotens.
 *
 * @see #isNull
 */
private
AvlNode( AvlNode p )
{
	left = right = null;
	height = -1;
	balance = 0;

	parent = p;
	key = 0;
	isNull = true;
}

/**
 * Liefert einen Wurzel-<code>NULL</code>-Knoten.
 *
 * <p>
 * Die beiden Konstruktoren sind <code>private</code>, also k&ouml;nnen
 * von einer Anwendung nur mit dieser Methode Knoten erzeugt werden.
 * </p>
 *
 * <p>
 * Da der von dieser Methode gelieferte Knoten ein Wurzelknoten ist,
 * gilt f&uuml;r den <code>parent p</code> des Knotens
 * <code>p.isNull == true</code>.
 * </p>
 *
 * @return Einen Wurzel-<code>NULL</code>-Knoten.
 */
public static AvlNode
createRootNode()
{
	return new AvlNode(new AvlNode());
}

/**
 * Liefert den Schl&uuml;ssel des Knotens.
 *
 * Darf nicht bei einem Knoten aufgerufen werden fuer den
 * <code>isNull == true</code> gilt.
 *
 * @return Den Schl&uuml;ssel des Knotens.
 *
 * @see #isNull
 * @see #setKey
 *
 * @throws Exception Wenn <code>getKey()</code> bei einem
 * 	<code>NULL</code>-Knoten aufgerufen wird.
 */
public int
getKey() throws Exception
{
	if (isNull) {
		throw new Exception("AvlNode#getKey() called on invalid node");
	}

	return key;
}

/**
 * Setzt den Schl&uuml;ssel des Knotens auf <code>newKey</code>.
 *
 * Falls <code>setKey</code> auf einen <code>NULL</code>-Knoten aufgerufen
 * wird, wird der Knoten zu einem regul&auml;ren Knoten mit zwei neuen
 * <code>NULL</code>-Knoten als Kindern.
 *
 * @param newKey Der neue Schl&uuml;ssel des Knotens.
 *
 * @see #isNull
 * @see #getKey
 */
public void
setKey( int newKey )
{
	key = newKey;

	if (isNull) {
		left = new AvlNode(this);
		right = new AvlNode(this);
		height = 0;
		isNull = false;
	}
}

/**
 * Liefert den Wert der read-only Eigenschaft {@link #isNull}.
 *
 * @return <code>true</code> wenn es sich um einen
 * 	<code>NULL</code>-Knoten handelt, <code>false</code> sonst.
 *
 * @see #isNull
 */
public boolean
isNull()
{
	return isNull;
}

/**
 * Setzt {@link #debugLevel} auf <code>d</code>.
 *
 * @param d Der neue Debug-Level. Wird ein ung&uuml;ltiger Wert
 *	&uuml;bergeben, wird <code>debugLevel</code> auf
 *	<code>DEBUG_ALL</code> gesetzt.
 *
 * @see #debugLevel
 */
public static void
setDebugLevel( int d )
{
	if (d > DEBUG_ALL) d = DEBUG_ALL;

	AvlNode.debugLevel = d;
}

/**
 * Liefert den aktuellen Debug-Level.
 *
 * @return Der aktuelle Wert von {@link #debugLevel}.
 *
 * @see #debugLevel
 */
public static int
getDebugLevel()
{
	return AvlNode.debugLevel;
}

/**
 * Liefert eine String-Repr&auml;sentation des Knotens abh&auml;ngig vom
 * aktuellen {@link #debugLevel Debug-Level}.
 *
 * @return Eine String-Repr&auml;sentation des Knotens.
 *
 * @see #debugLevel
 */
public String
toString()
{
	return toString(AvlNode.debugLevel);
}

/**
 * Liefert eine String-Repr&auml;sentation des Knotens mit dem gegebenen
 * Debug-Level <code>d</code>.
 *
 * @param d Der gew&uuml;nschte {@link #debugLevel Debug-Level}.
 * @return Eine String-Repr&auml;sentation des Knotens.
 *
 * @see #debugLevel
 */
public String
toString( int d )
{
	if (isNull) return "";
	if (d == DEBUG_OFF) return String.valueOf(key);

	StringBuffer ret = new StringBuffer("[key=" + key);

	if ((d & DEBUG_HEIGHT) != 0) ret.append(";height=" + height);

	if ((d & DEBUG_BALANCE) != 0) ret.append(";balance=" + balance);

	if ((d & DEBUG_CHILDREN) != 0) {
		ret.append(";left=");
		if (left != null) ret.append(left.toString(DEBUG_OFF));
		ret.append(";right=");
		if (right != null) ret.append(right.toString(DEBUG_OFF));
	}

	if ((d & DEBUG_PARENT) != 0) {
		ret.append(";parent=");

		if (parent == null) ret.append("NULL");
		else ret.append(parent.toString(DEBUG_OFF));
	}

	if ((d & DEBUG_ID) != 0) {
		String s = super.toString();

		ret.append(";id=" + s.substring(s.indexOf('@') + 1));
	}

	ret.append("]");

	return ret.toString();
}

} // class AvlNode
