package ads1ws08.pa1;

/**
 * AVL-Baum-Klasse die die fehlenden Methoden aus {@link AbstractAvlTree}
 * implementiert.
 *
 * <p>
 * In dieser Klasse m&uuml;ssen Sie Ihren Code einf&uuml;gen und die
 * Methoden {@link #remove remove()} sowie {@link #rotateLeft rotateLeft()}
 * und {@link #rotateRight rotateRight()} implementieren.
 * </p>
 *
 * <p>
 * Sie k&ouml;nnen beliebige neue Variablen und Methoden in dieser
 * Klasse hinzuf&uuml;gen. Wichtig ist nur, dass die oben genannten
 * Methoden implementiert werden.
 * </p>
 */
public class AvlTreeImpl extends AbstractAvlTree
{

	/**
	 * Der Default-Konstruktor.
	 *
	 * Ruft einfach nur den Konstruktor der Oberklasse auf.
	 */
	public AvlTreeImpl()
	{
		super();
	}

	public void insert(int k) throws Exception
	{
		super.insert(k);
	}

	/**
	 * Entfernt den Knoten mit Schl&uuml;ssel <code>k</code> falls
	 * er existiert.
	 *
	 * <p>
	 * Existiert im AVL-Baum kein Knoten mit Schl&uuml;ssel
	 * <code>k</code>, soll <code>remove()</code> einfach nichts
	 * machen.
	 * </p>
	 *
	 * <p>
	 * Nach dem Entfernen muss sichergestellt sein, dass es sich bei
	 * dem resultierenden Baum immer noch um einen AVL-Baum handelt,
	 * und dass {@link AbstractAvlTree#root root} auf die
	 * tats&auml;chliche Wurzel des Baums zeigt!
	 * </p>
	 *
	 * @param k Der Schl&uuml;ssel dessen Knoten entfernt werden soll.
	 *
	 * @see AbstractAvlTree#root
	 * @see #rotateLeft rotateLeft()
	 * @see #rotateRight rotateRight()
	 */
	public void remove(int k) throws Exception
	{
		if (root.isNull())
			return;
		root = remove(root, k);
	}

	private AvlNode remove(AvlNode n, int k) throws Exception
	{
		if (n.isNull())
			return n;

		// node is in left subtree
		if (k < n.getKey())
		{
			// remove it from left subtree
			n.left = remove(n.left, k);
			n.balance = calcBalance(n);
			n = rebalance(n);
		}
		// node is in right subtree
		else if (k > n.getKey())
		{
			// remove it from right subtree
			n.right = remove(n.right, k);
			n.balance = calcBalance(n);
			n = rebalance(n);
		}
		// found node, has leafs
		else if (!n.left.isNull() && !n.right.isNull())
		{
			// fetch replacement node from deeper subtree (may avoid a rotation later)
			if (n.left.height > n.right.height)
			{
				n.setKey(getMax(n.left).getKey());
				n.left = remove(n.left, n.getKey());
			}
			else
			{
				n.setKey(getMin(n.right).getKey());
				n.right = remove(n.right, n.getKey());
			}
		}
		// found node, no leafs
		else if (n.left.isNull() && n.right.isNull())
			return AvlNode.createRootNode();
		// found node, one leaf (either left or right)
		else
		{
			if (!n.left.isNull())
			{
				n.left.parent = n.parent;
				n = n.left;
			}
			else
			{
				n.right.parent = n.parent;
				n = n.right;
			}
		}

		// update height + balance
		n.height = Math.max(n.left.height, n.right.height) + 1;
		n.balance = calcBalance(n);

		return n;
	}

	public AvlNode getMax(AvlNode n) throws Exception
	{
		if (n.isNull())
			return n;
		while (!n.right.isNull())
			n = n.right;
		return n;
	}

	public AvlNode getMin(AvlNode n) throws Exception
	{
		if (n.isNull())
			return n;
		while (!n.left.isNull())
			n = n.left;
		return n;
	}

	public AvlNode rebalance(AvlNode n)
	{
		// fall 2
		// rotate right if right subtree is two deeper than left subtree
		if (n.balance == 2)
		{
			// fall 2.1
			if (n.right.balance == 0 || n.right.balance == 1)
			{
				n = rotateLeft(n);
				if (!n.left.right.isNull())
					n.left.right.parent = n.left;
			}
			// fall 2.2
			else
				n = rotateRightLeft(n);
		}
		// fall 1
		else if (n.balance == -2)
		{
			// fall 1.1
			if(n.left.balance == 0 || n.left.balance == -1)
			{
				n = rotateRight(n);
				if (!n.right.left.isNull())
					n.right.left.parent = n.right;
			}
			// fall 1.2
			else
				n = rotateLeftRight(n);
		}

		return n;
	}

	/**
	 * F&uuml;hrt eine Links-Rotation beim Knoten <code>n</code>
	 * durch.
	 *
	 * <p>
	 * <b>HINWEIS:</b> Beachten Sie, dass alle von Ihrer Implementierung
	 * <em>verwendeten</em> Eigenschaften von {@link AvlNode}
	 * (<code>parent</code>, <code>left</code>, <code>right</code>,
	 * <code>height</code>, <code>balance</code>) bei den von der
	 * Rotation betroffenen Knoten eventuell aktualisiert werden
	 * m&uuml;ssen.
	 * </p>
	 *
	 * @param n Der Knoten bei dem die Rotation durchgef&uuml;hrt
	 * 	werden soll.
	 *
	 * @return Die <em>neue</em> Wurzel des rotierten Teilbaums.
	 */
	public AvlNode rotateLeft(AvlNode n)
	{
		AvlNode m = n.right;
		n.right = m.left;
		m.left = n;

		m.parent = n.parent;
		n.parent = m;
		if (!n.right.isNull())
			n.right.parent = n;

		n.height = Math.max(n.left.height, n.right.height) + 1;
		m.height = Math.max(m.right.height, n.height) + 1;

		n.balance = calcBalance(n);
		m.balance = calcBalance(m);

		return m;
	}

	/**
	 * F&uuml;hrt eine Rechts-Rotation beim Knoten <code>n</code>
	 * durch.
	 *
	 * <p>
	 * <b>HINWEIS:</b> Beachten Sie, dass alle von Ihrer Implementierung
	 * <em>verwendeten</em> Eigenschaften von {@link AvlNode}
	 * (<code>parent</code>, <code>left</code>, <code>right</code>,
	 * <code>height</code>, <code>balance</code>) bei den von der
	 * Rotation betroffenen Knoten eventuell aktualisiert werden
	 * m&uuml;ssen.
	 * </p>
	 *
	 * @param n Der Knoten bei dem die Rotation durchgef&uuml;hrt
	 * 	werden soll.
	 *
	 * @return Die <em>neue</em> Wurzel des rotierten Teilbaums.
	 */
	public AvlNode rotateRight(AvlNode n)
	{
		AvlNode m = n.left;
		n.left = m.right;
		m.right = n;

		m.parent = n.parent;
		n.parent = m;
		if (!n.left.isNull())
			n.left.parent = n;

		n.height = Math.max(n.left.height, n.right.height) + 1;
		m.height = Math.max(m.left.height, n.height) + 1;

		n.balance = calcBalance(n);
		m.balance = calcBalance(m);

		return m;
	}

	public int calcBalance(AvlNode n)
	{
		int l = (!n.left.isNull())  ? n.left.height  + 1 : 0;
		int r = (!n.right.isNull()) ? n.right.height + 1 : 0;
		return r - l;
	}

	public AvlNode rotateLeftRight(AvlNode n)
	{
		n.left = rotateLeft(n.left);
		return rotateRight(n);
	}

	public AvlNode rotateRightLeft(AvlNode n)
	{
		n.right = rotateRight(n.right);
		return rotateLeft(n);
	}
}
