package ads1ws08.pa1;

import java.io.FileInputStream;

/**
 * Diese Klasse enth&auml;lt nur die {@link #main main()}-Methode zum starten
 * des Programms.
 *
 * <p>
 * <b>WICHTIG:</b>Nehmen Sie keine &Auml;nderungen in dieser Klasse vor.
 * Bei der Abgabe werden diese &Auml;nderungen verworfen und es k&ouml;nnte
 * dadurch passieren, dass Ihr Programm somit nicht mehr korrekt
 * funktioniert.
 * </p>
 */
public class Main
{

private static void
bailOut( String msg )
{
	System.out.println("#error " + msg);
	System.exit(0);
}

/**
 * Lie&szlig;t die Daten einer Testinstanz ein und &uuml;bergibt sie
 * an die entsprechenden Methoden der AVL-Baum implementation.
 *
 * <p>
 * Wenn auf der Kommandozeile die Option <code>-v</code> angegeben wird,
 * werdem zu jedem ausgegebenen Knoten detaillierte Informationen
 * ausgegeben. Um den Detailgrad zu erh&ouml;hen, kann die Option
 * <code>-v</code> mehrmals angegeben werden, oder man h&auml;ngt nach
 * einem ':' eine Liste von Informationen an die ausgegeben werden
 * sollen. Die m&ouml;glichen Parameter f&uuml;r <code>-v:</code> sind:
 * <ul>
 *   <li>{@link AvlNode#DEBUG_CHILDREN children}</li>
 *   <li>{@link AvlNode#DEBUG_HEIGHT height}</li>
 *   <li>{@link AvlNode#DEBUG_BALANCE balance}</li>
 *   <li>{@link AvlNode#DEBUG_PARENT parent}</li>
 *   <li>{@link AvlNode#DEBUG_ID id}</li>
 * </ul>
 * So werden durch die Option <code>-v:parent,children</code> zu jedem
 * Knoten dessen Kinder und der Elternknoten ausgegeben (keine
 * Leerzeichen zwischen dem ':' und den ',').
 * </p>
 *
 * <p>
 * Der erste String in <code>args</code>, der <em>nicht</em> mit
 * <code>-v</code> beginnt, wird als der Name der Datei interpretiert,
 * aus der die Testinstanz auszulesen ist. Wird kein Dateiname angegeben,
 * wird die Testinstanz &uuml;ber {@link System#in} eingelesen.
 * </p>
 *
 * <p>
 * M&ouml;glicherweise auftretende Exceptions werden auf
 * {@link System#err} ausgegeben.
 * </p>
 * 
 * @param args Die von der Kommandozeile &uuml;bergebene Argumente.
 * 	Die Option <code>-v</code> aktiviert detaillierte
 * 	Knoteninformationen (kann mehrfach verwendet werden). Der
 * 	erste andere String wird als Dateiname interpretiert.
 */
public static void
main( String[] args )
{
	String fileName = null;
	Scanner scanner = null;
	AbstractAvlTree tree = new AvlTreeImpl();

	for (String a: args) {
		if (a.equals("-v")) {
			AvlNode.setDebugLevel((AvlNode.getDebugLevel()<<1) + 1);
		} else if (a.startsWith("-v:")) {
			String[] ds = a.substring(3).split(",");
			int dl = AvlNode.getDebugLevel();

			for (String d: ds) {
				if (d.equals("children")) {
					dl |= AvlNode.DEBUG_CHILDREN;
				} else if (d.equals("height")) {
					dl |= AvlNode.DEBUG_HEIGHT;
				} else if (d.equals("balance")) {
					dl |= AvlNode.DEBUG_BALANCE;
				} else if (d.equals("parent")) {
					dl |= AvlNode.DEBUG_PARENT;
				} else if (d.equals("id")) {
					dl |= AvlNode.DEBUG_ID;
				} else if (d.equals("m1")) {
					dl |= AvlNode.DEBUG_CHILDREN | AvlNode.DEBUG_HEIGHT | AvlNode.DEBUG_BALANCE | AvlNode.DEBUG_PARENT;
				}
			}

			AvlNode.setDebugLevel(dl);
		} else if (fileName == null) {
			fileName = a;
		}
	}

	if (fileName != null) try {
		scanner = new Scanner(new FileInputStream(fileName));
	} catch (Exception e) {
		// Nothing to do, this will be handled later.
	}

	if (scanner == null) scanner = new Scanner(System.in);

	String errorMessage;
	try {
		while (scanner.nextValue()) {
			int v = scanner.getValue();

			switch (scanner.getType()) {
				case INVALID:
					throw new Exception(
						"Invalid scanner access");
				case INSERT:
					tree.insert(v);
					if (tree.find(v).isNull()) {
						bailOut("inserting node " + v + " - node not found in tree");
					}
					errorMessage = tree.testHeightBallance();
					if (errorMessage != "") {
						bailOut("inserting node: "+v+" - "+errorMessage);
					}
					break;
				case REMOVE:
					tree.remove(v);
					if (!tree.find(v).isNull()) {
						bailOut("removing node " + v + " - node still found in tree");
					}
					errorMessage = tree.testHeightBallance();
					if (errorMessage != "") {
						bailOut("remove node: "+v+" - "+errorMessage);
					}
					break;
				case FIND:
					tree.find(v);
					break;
				default:
					assert false
						: "This should be unreachable";
			}

			if (scanner.debugRequested()) tree.printDebug();
		}

		System.out.println(tree.getResult());
	} catch (Exception e) {
		e.printStackTrace();
	}
} // void main()

} // class Main
