package ads1ws08.pa1.viewer;

import java.awt.Dimension;
import java.awt.Graphics;
import java.awt.Color;
import java.awt.Canvas;
import java.awt.Font;
import java.awt.FontMetrics;
import java.util.List;
import java.util.Iterator;
import java.util.HashMap;

public final class TreeCanvas extends Canvas
{

private static final long serialVersionUID = 1L;

private final int DIAMETER = 30;
private final int DIA_H = DIAMETER >> 1;
private final int DIA_Q = DIAMETER >> 2;
private final int DIA_D = DIAMETER << 1;

private TreeRepresentation tree;

public
TreeCanvas()
{
	this(null);
}

public
TreeCanvas( TreeRepresentation tree )
{
	this.tree = tree;

	setForeground(Color.BLACK);
	setBackground(Color.WHITE);
}

public boolean
setTree( TreeRepresentation tree )
{
	if (tree == null || tree == this.tree) {
		System.err.println("No or the same tree");

		return false;
	}

	this.tree = tree;

	repaint();

	return true;
}

public void
update( Graphics g )
{
	super.update(g);
	paint(g);
}

public void
paint( Graphics g )
{
	if (tree == null) {
		System.err.println("No tree");

		return;
	}

	paintTree(g, getWidth() / 2, DIAMETER, tree.height,
		tree.inorder, tree.preorder.iterator(), tree.nodeinfos);

	setPreferredSize(
		new Dimension(exp2(tree.height + 2) * DIAMETER - DIAMETER,
		(tree.height + 2) * DIA_D));
}

private int
exp2( int exp )
{
	if (exp == 0) return 1;

	return 2 << (exp - 1);
}

private void
paintTree( Graphics g, int x, int y, int height,
		List<Integer> io, Iterator<Integer> po, HashMap<Integer, HashMap<String, String>> nodeinfos)
{
	Integer i = po.next();
	int r = io.indexOf(i);
	int l  = r + 1;
	g.setFont(g.getFont().deriveFont(Font.BOLD));
	FontMetrics fm = g.getFontMetrics();

	g.drawLine(x, y, x - exp2(height) * DIAMETER, y + DIA_D);
	g.drawLine(x, y, x + exp2(height) * DIAMETER, y + DIA_D);

	g.setColor(Color.WHITE);
	g.fillOval(x - DIA_H, y - DIA_H, DIAMETER, DIAMETER);
	g.setColor(Color.BLACK);
	g.drawOval(x - DIA_H, y - DIA_H, DIAMETER, DIAMETER);

	g.setColor(Color.blue);
	g.drawString(i.toString(), x - DIA_Q - fm.stringWidth(i.toString())/2+fm.charWidth('0'), y + DIA_Q);
	String info = "";
	if (nodeinfos.containsKey(i))
	{
		if (nodeinfos.get(i).containsKey("balance"))
			info += "b="+nodeinfos.get(i).get("balance");
		if (nodeinfos.get(i).containsKey("height"))
		{
			if (info.length() > 0)
				info += ", ";
			info += "h="+nodeinfos.get(i).get("height");
		}
		if (nodeinfos.get(i).containsKey("parent"))
		{
			if (info.length() > 0)
				info += ", ";
			info += "p="+nodeinfos.get(i).get("parent");
		}
	}
	if (info.length() > 0)
	{
		g.setColor(Color.red);
		g.drawString(info, x - DIA_Q + DIAMETER, y + DIA_Q+DIAMETER/2);
	}

	g.setColor(Color.BLACK);
	if (r > 0) {
		paintTree(g, x - exp2(height) * DIAMETER, y + DIA_D,
			height - 1, io.subList(0, r), po, nodeinfos);
	} else {
		g.fillRect(x - DIA_Q - exp2(height) * DIAMETER,
			y - DIA_Q + DIA_D, DIA_H, DIA_H);
	}

	if (l < io.size()) {
		paintTree(g, x + exp2(height) * DIAMETER, y + DIA_D,
			height - 1, io.subList(l, io.size()), po, nodeinfos);
	} else {
		g.fillRect(x - DIA_Q + exp2(height) * DIAMETER,
			y - DIA_Q + 2 * DIAMETER, DIA_H, DIA_H);
	}
}

} // class TreeCanvas
