package ads1ws08.pa1.viewer;

import java.io.InputStream;
import java.io.InputStreamReader;
import java.io.BufferedReader;
import java.util.ArrayList;
import java.util.LinkedList;
import java.util.ListIterator;
import java.util.HashMap;

public final class TreeReader
{

private BufferedReader input;
private String lastTree;
private LinkedList<TreeRepresentation> trees;
private ListIterator<TreeRepresentation> treesIterator;
private TreeRepresentation currentTree;
private boolean lastWasForward;

public
TreeReader() throws Exception
{
	this(System.in);
}

public
TreeReader( InputStream in ) throws Exception
{
	input = new BufferedReader(new InputStreamReader(in));
	trees = new LinkedList<TreeRepresentation>();

	for (String s = input.readLine(); s != null; s = input.readLine()) {
		if (!s.startsWith("#tree") || s.equals(lastTree)) continue;

		lastTree = s;

		String[] tokens = s.split(" ");

		if (tokens.length != 4
				|| tokens[2].length() != tokens[3].length()) {
			throw new Exception("TreeReader: invalid "
				+ "input format");
		}

		trees.add(new TreeRepresentation(Integer.valueOf(tokens[1]),
			createList(tokens[2].split(",")),
			createList(tokens[3].split(",")),
			createInfos(tokens[2].split(","))));
	}

	treesIterator = trees.listIterator();
	lastWasForward = true;
	currentTree = null;
}

private ArrayList<Integer>
createList( String[] a )
{
	ArrayList<Integer> ret = new ArrayList<Integer>();

	for (String s: a) {
		if (s.length() == 0) continue;

		if (s.charAt(0) != '[') {
			ret.add(Integer.valueOf(s));
		} else {
			ret.add(Integer.valueOf(
				s.substring(5, s.indexOf(';'))));
		}
	}

	ret.trimToSize();

	return ret;
}

private HashMap<Integer, HashMap<String, String>>
createInfos( String[] a )
{
	HashMap<Integer, HashMap<String, String>> ret = new HashMap<Integer, HashMap<String, String>>();

	for (String s: a) {
		if (s.length() == 0) continue;

		if (s.charAt(0) == '[' && s.length() > 2) {
			HashMap<String, String> tmp = new HashMap<String, String>();
			int nodeval = Integer.valueOf(s.substring(5, s.indexOf(';')));
			String nodeinfos[] = s.substring(1, s.length() - 1).split(";");
			for(int i=0; i < nodeinfos.length; i++)
			{
				String keyval[] = nodeinfos[i].split("=");
				tmp.put(keyval[0], (keyval.length == 1) ? null : keyval[1]);
			}
			ret.put(nodeval, tmp);
		}
	}

	return ret;
}

public TreeRepresentation
getFirstTree()
{
	treesIterator = trees.listIterator();
	currentTree = treesIterator.next();
	lastWasForward = true;

	return currentTree;
}

public TreeRepresentation
getNextTree()
{
	if (treesIterator.hasNext()) {
		currentTree = treesIterator.next();
		if (!lastWasForward) currentTree = treesIterator.next();
	}

	lastWasForward = true;

	return currentTree;
}

public TreeRepresentation
getPreviousTree()
{
	if (treesIterator.hasPrevious()) {
		currentTree = treesIterator.previous();
		if (lastWasForward) currentTree = treesIterator.previous();
	}

	lastWasForward = false;

	return currentTree;
}

public TreeRepresentation
getLastTree()
{
	treesIterator = trees.listIterator(trees.size() - 1);
	currentTree = trees.getLast();

	lastWasForward = false;

	return currentTree;
}

public int
numTrees()
{
	return trees.size();
}

} // class TreeReader
