package ads1ws08.pa1.viewer;

import java.awt.Label;
import java.awt.Panel;
import java.awt.ScrollPane;
import java.awt.Button;
import java.awt.BorderLayout;
import java.awt.GridLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

public final class TreeViewer extends Panel
{

private static final long serialVersionUID = 1L;

private TreeReader trees;
private TreeCanvas canvas;
private Button first;
private Button prev;
private Button next;
private Button last;
private Label navLabel;
private int step;

public
TreeViewer() throws Exception
{
	this(new TreeReader());
}

public
TreeViewer( TreeReader reader )
{
	super(new BorderLayout());

	Panel navigation = new Panel();
	Panel buttons = new Panel();
	GridLayout navLayout = new GridLayout(2, 1);
	ScrollPane canvasPane = new ScrollPane();

	trees = reader;

	canvas = new TreeCanvas();

	step = 1;
	navLabel = new Label();
	navLabel.setAlignment(Label.CENTER);
	updateStepLabel();

	first = new Button("|<");
	prev = new Button("<<");
	next = new Button(">>");
	last = new Button(">|");

	first.addActionListener(new ActionListener() {
		public void actionPerformed(ActionEvent e) {
			canvas.setTree(trees.getFirstTree());
			step = 1;
			updateStepLabel();
		}
	});

	prev.addActionListener(new ActionListener() {
		public void actionPerformed(ActionEvent e) {
			if (canvas.setTree(trees.getPreviousTree())) {
				step--;
				updateStepLabel();
			}
		}
	});

	next.addActionListener(new ActionListener() {
		public void actionPerformed(ActionEvent e) {
			if (canvas.setTree(trees.getNextTree())) {
				step++;
				updateStepLabel();
			}
		}
	});

	last.addActionListener(new ActionListener() {
		public void actionPerformed(ActionEvent e) {
			canvas.setTree(trees.getLastTree());
			step = trees.numTrees();
			updateStepLabel();
		}
	});

	canvasPane.add(canvas);

	buttons.add(first);
	buttons.add(prev);
	buttons.add(next);
	buttons.add(last);

	navigation.setLayout(navLayout);
	navigation.add(navLabel);
	navigation.add(buttons);

	add(canvasPane, BorderLayout.CENTER);
	add(navigation, BorderLayout.SOUTH);

	canvas.setTree(trees.getFirstTree());
}

public
TreeViewer( TreeReader reader, String fileName )
{
	this(reader);

	Label l = new Label("File: " + fileName);

	l.setAlignment(Label.CENTER);

	add(l, BorderLayout.NORTH);
}

private void
updateStepLabel()
{
	navLabel.setText("Step " + step + " of " + trees.numTrees());
}

} // class TreeViewer
